@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 ###
-### $Release: 0.0.2 $
+### $Release: 0.0.3 $
 ### copyright(c) 2008-2009 kuwata-lab.com all rights reserved.
 ### MIT License
 ###
@@ -10,9 +10,11 @@ import sys, os, re
 import shutil
 #from glob import glob as _glob
 import kook
-from kook import _report_cmd, KookCommandError
+from kook import KookCommandError
 from kook.utils import *
 from kook.utils import glob2
+import kook.config as config
+from kook.misc import _report_cmd
 
 
 __all__ = (
@@ -30,14 +32,20 @@ __all__ = (
 
 def system_f(command):
     _report_cmd(command)
+    if config.noexec:
+        return
     return os.system(command)
 
 
 def system(command):
     _report_cmd(command)
+    if config.noexec:
+        return
     status = os.system(command)
     if status != 0:
+        #status = status % 255
         raise KookCommandError("command failed (status=%s)." % status)
+    return status
 
 
 def cp(*filenames):
@@ -61,6 +69,8 @@ def _prepare(filenames, cmd=None):
 
 def _cp(filenames, func, cmd, p, r):
     fnames = _prepare(filenames, cmd)
+    if config.noexec:
+        return
     _copy_file = p and shutil.copy2 or shutil.copy
     n = len(fnames)
     if n < 2:
@@ -122,6 +132,8 @@ def mkdir_p(*dirnames):
 
 def _mkdir(dirnames, func, cmd, p):
     dnames = _prepare(dirnames, cmd)
+    if config.noexec:
+        return
     if not dnames:
         raise KookCommandError("%s: directory name required." % func)
     for dname in dnames:
@@ -161,6 +173,8 @@ def rm_rf(*filenames):
 
 def _rm(filenames, func, cmd, r=False, f=False):
     fnames = _prepare(filenames, cmd)
+    if config.noexec:
+        return
     for fname in fnames:
         if os.path.isdir(fname):
             if r: _remove(fname)
@@ -186,6 +200,8 @@ def mv(*filenames):
 
 def _mv(filenames, func, cmd):
     fnames = _prepare(filenames, cmd)
+    if config.noexec:
+        return
     n = len(fnames)
     if n < 2:
         raise KookCommandError("%s: at least two file or directory names are required." % func)
@@ -220,22 +236,26 @@ def echo_n(*messages):
 
 def _echo(messages, func, cmd, n=False):
     msgs = _prepare(messages, cmd)
-    write = kook._stdout.write
+    if config.noexec:
+        return
+    write = config.stdout.write
     for i, msg in enumerate(msgs):
         if i > 0: write(' ')
         write(msg)
     if not n: write("\n")
-    kook._stdout.flush()      # requires for 'echo -n'
+    config.stdout.flush()      # requires for 'echo -n'
 
 
 def store(*filenames):
     _store(filenames, 'store', 'store', p=False)
 
 def store_p(*filenames):
-    _store(filenames, 'store', 'store', p=True)
+    _store(filenames, 'store_p', 'store -p', p=True)
 
 def _store(filenames, func, cmd, p=False):
     fnames = _prepare(filenames, cmd)
+    if config.noexec:
+        return
     n = len(fnames)
     _copy_file = p and shutil.copy2 or shutil.copy
     if n < 2:
@@ -281,8 +301,9 @@ class Chdir(object):
             raise KookCommandError("%s: %s: not a directory." % (func, dname))
         self.cwd = os.getcwd()
         os.chdir(self.dirname)
+        return self
 
-    def __exit__(self, type, value, traceback):
+    def __exit__(self, type, value, traceback):   # type and value is not used
         _prepare(['-', '  # back to '+self.cwd], self.cmd)
         os.chdir(self.cwd)
 
@@ -330,6 +351,8 @@ def edit(*filenames, **kwargs):
 
 def _edit(filenames, func, cmd, by, encoding=None):
     fnames = _prepare(filenames, cmd)
+    if config.noexec:
+        return
     for fname in fnames:
         if not os.path.exists(fname):
             raise KookCommandError("%s: %s: not found." % (func, fname))
