@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 ###
-### $Release: 0.0.2 $
+### $Release: 0.0.3 $
 ### copyright(c) 2008-2009 kuwata-lab.com all rights reserved.
 ### MIT License
 ###
@@ -10,11 +10,35 @@
 #from kook.kitchen import IfExists
 from kook.utils import flatten
 
-__all__ = ('product', 'ingreds', 'byprods', 'coprods', 'priority', 'spices', 'cmdopts', 'if_exists', )
+__all__ = ('recipe', 'product', 'ingreds', 'byprods', 'coprods', 'priority', 'spices', 'cmdopts', )
+
+
+def recipe(func):
+    func._kook_recipe = True
+    return func
+
+#def recipe(func=None, kind=None):
+#    if func is not None:
+#        func._kook_recipe = True
+#        return func
+#    elif kind is not None:
+#        if kind not in ('task', 'file'):
+#            raise TypeError("@recipe(): kind should be 'task' or 'file'.")
+#        def deco(f):
+#            f._kook_recipe = True
+#            f._kook_kind = kind
+#            return f
+#        return deco
+#    else:
+#        def deco(f):
+#            f._kook_recipe = True
+#            return f
+#        return deco
 
 
 def product(name):
     def deco(f):
+        #f._kook_recipe  = True
         f._kook_product = name
         return f
     return deco
@@ -22,27 +46,27 @@ def product(name):
 
 def ingreds(*names):
     def deco(f):
-        f._kook_ingreds = flatten(names)
+        f._kook_ingreds = tuple(flatten(names))
         return f
     return deco
 
 
 def byprods(*names):
     def deco(f):
-        f._kook_byprods = flatten(names)
+        f._kook_byprods = tuple(flatten(names))
         return f
     return deco
 
 
 def coprods(*names):
     def deco(f):
-        f._kook_coprods = flatten(names)
+        f._kook_coprods = tuple(flatten(names))
         return f
     return deco
 
 
-def priority(level):
-    if not isinstance(leve, int):
+def priority(level):     # not used yet
+    if not isinstance(level, int):
         import kook
         raise kook.KookRecipeError("priority requires integer.")  # TODO: change backtrace
     def deco(f):
@@ -53,7 +77,7 @@ def priority(level):
 
 def spices(*names):
     def deco(f):
-        f._kook_spices = names
+        f._kook_spices = tuple(flatten(names))
         return f
     return deco
 
@@ -63,9 +87,3 @@ def cmdopts(*names):
     sys.stderr.write("[pykook] ERROR: '@cmdopts()' is obsolete. Use '@spices()' instead.\n")
     sys.stderr.write("[pykook]        See http://www.kuwata-lab.com/kook/pykook-CHANGES.txt for details.\n")
     sys.exit(1)
-
-
-def if_exists(*args):
-    import kook
-    #return kook.kitchen.IfExists(*args)
-    return [ kook.kitchen.IfExists(arg) for arg in flatten(args) ]
