@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 ###
-### $Release: 0.0.2 $
+### $Release: 0.0.3 $
 ### copyright(c) 2008-2009 kuwata-lab.com all rights reserved.
 ### MIT License
 ###
@@ -192,9 +192,9 @@ class CommandOptionParser(object):
     def __init__(self, optdef_strs=()):
         self.parse_spices(optdef_strs)
 
-    @classmethod
-    def new(cls, optdef_strs=()):
-        return cls(optdef_strs)
+    #@classmethod
+    #def new(cls, optdef_strs=()):
+    #    return cls(optdef_strs)
 
     def parse_spices(self, optdef_strs):
         helps = []
@@ -233,7 +233,7 @@ class CommandOptionParser(object):
         opts, longopts, rests = self._parse(cmd_args, command, check_longopts=False)
         return opts, longopts, rests
 
-    def _parse(self, cmd_args, command=None, check_longopts=True):
+    def _parse(self, cmd_args, command=None, check_longopts=True): ## command is currently not used
         spices = self.spices
         i = 0
         N = len(cmd_args)
@@ -244,6 +244,7 @@ class CommandOptionParser(object):
             if cmd_arg == '--':
                 i += 1
                 break
+            ## long opts
             m = re.match(r'^--([a-zA-Z][-\w]+)(?:=(.*))?$', cmd_arg)
             if m:
                 name, arg = m.group(1), m.group(2)
@@ -254,11 +255,11 @@ class CommandOptionParser(object):
                     raise CommandOptionError("%s: unknown command option." % cmd_arg)
                 elif spices[name] is False:    # --name
                     if arg is not None:
-                        raise CommandOptionError("%s: argument is now allowed." % cmd_arg)
+                        raise CommandOptionError("%s: argument is not allowed." % cmd_arg)
                     opts[name] = True
                 elif spices[name] is True:     # --name[=arg]
                     opts[name] = arg is None and True or arg
-                elif spices[name] is 1:        # --name[=N]
+                elif spices[name] == 1:        # --name[=N]
                     if arg and str2int(arg) is None:
                         raise CommandOptionError("%s: integer required." % cmd_arg)
                     opts[name] = arg is None and True or str2int(arg)
@@ -271,6 +272,7 @@ class CommandOptionParser(object):
                             raise CommandOptionError("%s: integer required." % cmd_arg)
                         arg = str2int(arg)
                     opts[name] = arg
+            ## short opts
             elif cmd_arg and cmd_arg[0] == '-':
                 optchars = cmd_arg
                 j = 1
@@ -286,7 +288,7 @@ class CommandOptionParser(object):
                     elif spices[ch] is True:   # -x[arg]
                         opts[ch] = optchars[j+1:] or True
                         break
-                    elif spices[ch] is 1:      # -x[N]
+                    elif spices[ch] == 1:      # -x[N]
                         arg = optchars[j+1:]
                         if arg and str2int(arg) is None:
                             raise CommandOptionError("-%s%s: integer required." % (ch, arg))
@@ -308,6 +310,7 @@ class CommandOptionParser(object):
                             arg = str2int(arg)
                         opts[ch] = arg
                         break
+            ## not an option
             else:
                 break
             i += 1
@@ -315,7 +318,7 @@ class CommandOptionParser(object):
         rests = cmd_args[i:]
         return check_longopts and (opts, rests) or (opts, longopts, rests)
 
-    def help(self, command=None, format="  %-20s: %s\n"):
+    def help(self, command=None, format="  %-20s: %s\n"): ## command is currently not used
         return "".join( format % (optdef_str, desc) for optdef_str, desc in self.helps if desc )
 
 
@@ -389,17 +392,41 @@ class CommandOptionParser(object):
 #
 
 
+def has_same_content(filename1, filename2, blocksize=1*1024*1024):   # 1MB
+    assert os.path.exists(filename1)
+    assert os.path.exists(filename2)
+    if os.path.getsize(filename1) != os.path.getsize(filename2):
+        return False
+    f1 = open(filename1, "rb")
+    try:
+        f2 = open(filename2, "rb")
+        try:
+            while True:
+                s1 = f1.read(blocksize)
+                s2 = f2.read(blocksize)
+                if not s1 and not s2: return True
+                if s1 != s2:          return False
+        finally:
+            f2.close()
+    finally:
+        f1.close()
+
+
 ## compatibility for Python 2.x and 3.0
 
 if python2:
     def _is_str(obj):
         return isinstance(obj, (str, unicode))  # unicode is not defined in Python 3.0
-    def _get_codeobj(func):
-        return func.func_code
+    def get_funcname(func):
+        return func.func_name
+    def get_funclineno(func):
+        return func.func_code.co_firstlineno
 elif python3:
     def _is_str(obj):
-        return isinstance(obj, str)
-    def _get_codeobj(func):
-        return func.__code__
+        return isinstance(obj, (str, bytes))
+    def get_funcname(func):
+        return func.__code__.co_name
+    def get_funclineno(func):
+        return func.__code__.co_firstlineno
 
 
